<?php

namespace Drupal\graph_mail\Plugin\Mail;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\ConfigFormBaseTrait;
use Drupal\Core\Logger\LoggerChannelTrait;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Logger\RfcLogLevel;
use Drupal\Core\Mail\MailInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\StringTranslation\TranslationInterface;
use Drupal\Core\Utility\Error;
use GuzzleHttp\ClientInterface;
use Microsoft\Graph\Graph;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Mime\Address;

/**
 * Defines the Microsoft Graph API Mail plugin.
 *
 * @Mail(
 *   id = "graphmail",
 *   label = @Translation("Microsoft Graph API Mail"),
 *   description = @Translation("Allows to send mail via the Microsoft Graph API.")
 * )
 */
class GraphMail implements MailInterface, ContainerFactoryPluginInterface {

  use ConfigFormBaseTrait;
  use LoggerChannelTrait;
  use StringTranslationTrait;

  /**
   * The configuration factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The HTTP client instance.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * Creates a \Drupal\graph_mail\Plugin\Mail\GraphMail object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param array $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The configuration factory service.
   * @param \Drupal\Core\StringTranslation\TranslationInterface $string_translation
   *   The string translation service.
   * @param \GuzzleHttp\ClientInterface $http_client
   *   The HTTP client instance.
   */
  // phpcs:ignore Generic.CodeAnalysis.UnusedFunctionParameter
  public function __construct(array $configuration, $plugin_id, $plugin_definition, LoggerChannelFactoryInterface $logger_factory = NULL, ConfigFactoryInterface $config_factory = NULL, TranslationInterface $string_translation = NULL, ClientInterface $http_client = NULL) {
    if (empty($logger_factory)) {
      @trigger_error('Calling GraphMail::__construct() without the $logger_factory argument is deprecated in graph_mail:1.0.0 and it will be required in graph_mail:2.0.0. See https://www.drupal.org/project/graph_mail/issues/3324314', E_USER_DEPRECATED);

      $logger_factory = \Drupal::service('logger.factory');
    }

    if (empty($config_factory)) {
      @trigger_error('Calling GraphMail::__construct() without the $config_factory argument is deprecated in graph_mail:1.0.0 and it will be required in graph_mail:2.0.0. See https://www.drupal.org/project/graph_mail/issues/3324314', E_USER_DEPRECATED);

      $config_factory = \Drupal::configFactory();
    }

    if (empty($string_translation)) {
      @trigger_error('Calling GraphMail::__construct() without the $string_translation argument is deprecated in graph_mail:1.0.0 and it will be required in graph_mail:2.0.0. See https://www.drupal.org/project/graph_mail/issues/3325160', E_USER_DEPRECATED);

      $string_translation = \Drupal::translation();
    }

    if (empty($http_client)) {
      @trigger_error('Calling GraphMail::__construct() without the $http_client argument is deprecated in graph_mail:1.0.0 and it will be required in graph_mail:2.0.0. See https://www.drupal.org/project/graph_mail/issues/3325211', E_USER_DEPRECATED);

      $http_client = \Drupal::httpClient();
    }

    $this->configFactory = $config_factory;
    $this->setLoggerFactory($logger_factory);
    $this->setStringTranslation($string_translation);
    $this->httpClient = $http_client;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('logger.factory'),
      $container->get('config.factory'),
      $container->get('string_translation'),
      $container->get('http_client')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function format(array $message) {
    return $message;
  }

  /**
   * {@inheritdoc}
   */
  public function mail(array $message) {
    try {
      $config = $this->config('graph_mail.mail_service.settings');

      $response = $this->httpClient->request(
        "POST",
        'https://login.microsoftonline.com/' . $config->get('tenant_id') . '/oauth2/v2.0/token',
        [
          'form_params' => [
            'client_id' => $config->get('client_id'),
            'client_secret' => $config->get('client_secret'),
            'scope' => 'https://graph.microsoft.com/.default',
            'grant_type' => 'client_credentials',
          ],
        ]
      );

      if ($response->getStatusCode() == 200 && $obj = json_decode($response->getBody()->getContents())) {
        $graph = (new Graph())
          ->setAccessToken($obj->access_token);

        if ($version = $config->get('version')) {
          $graph->setApiVersion($version);
        }

        $content = '';
        if (empty($message['body'])) {
          $message['body'] = $message['params']['message'];
        }
        if (!empty($message['body'])) {
          if (is_string($message['body']) || is_a($message['body'], MarkupInterface::class)) {
            $content = (string) $message['body'];
          }
          elseif (is_array($message['body'])) {
            $content = reset($message['body']);
          }
        }

        if (empty($message['subject'])) {
          $message['subject'] = $message['params']['subject'];
        }

        $mail_body = [
          'message' => [
            'subject' => $message['subject'],
            'body' => [
              'contentType' => 'HTML',
              'content' => $content,
            ],
            'from' => [
              'emailAddress' => [
                'address' => $config->get('default_mail') ?? $this->config('system.site')->get('mail'),
              ],
            ],
            'toRecipients' => [],
            'ccRecipients' => [],
            'bccRecipients' => [],
            'replyTo' => [],
          ],
          'saveToSentItems' => $config->get('save_to_sent_items') ? 'true' : 'false',
        ];

        foreach (explode(',', $message['to']) as $to) {
          $mail_body['message']['toRecipients'][] = [
            'emailAddress' => $this->parseEmailAddress(trim($to)),
          ];
        }

        if (!empty($message['headers'])) {
          $headers = array_change_key_case($message['headers']);
          if (!empty($headers['bcc'])) {
            foreach (explode(',', $headers['bcc']) as $address) {
              $mail_body['message']['bccRecipients'][] = [
                'emailAddress' => $this->parseEmailAddress(trim($address)),
              ];
            }
          }
          if (!empty($headers['cc'])) {
            foreach (explode(',', $headers['cc']) as $address) {
              $mail_body['message']['ccRecipients'][] = [
                'emailAddress' => $this->parseEmailAddress(trim($address)),
              ];
            }
          }
          if (!empty($headers['reply-to'])) {
            foreach (explode(',', $headers['reply-to']) as $address) {
              $mail_body['message']['replyTo'][] = [
                'emailAddress' => [
                  'address' => $this->parseEmailAddress(trim($address)),
                ],
              ];
            }
          }
        }

        if (isset($message['params']['attachments'])) {
          $files = $message['params']['attachments'];
          foreach ($files as $file) {
            $mail_body['message']['attachments'][] = [
              '@odata.type' => '#microsoft.graph.fileAttachment',
              'name' => $file['filename'],
              'contentBytes' => base64_encode($file['filecontent']),
            ];
          }
        }

        $graph->createRequest('POST', '/users/' . $config->get('user_id') . '/sendmail')
          ->attachBody($mail_body)
          ->execute();

        return TRUE;
      }
      else {
        return FALSE;
      }
    }
    catch (\Exception $e) {
      $this->getLogger('graph_mail')
        ->log(RfcLogLevel::ERROR, '%type: @message in %function (line %line of %file).', Error::decodeException($e));

      return FALSE;
    }
  }

  /**
   * Parses the e-mail address to comply to the emailAddress resource type.
   *
   * @param string $email_address
   *   The e-mail address to parse.
   *
   * @return array
   *   An array containing the parsed e-mail address.
   *
   * @see https://learn.microsoft.com/en-us/graph/api/resources/emailaddress?view=graph-rest-1.0
   */
  protected function parseEmailAddress(string $email_address): array {
    $address = Address::create($email_address);

    return [
      'address' => $address->getAddress(),
      'name' => $address->getName(),
    ];
  }

}
